import mcpi

from .object import Object
from .wall import Wall


class TownWall(Object):

    height = 3
    wall_space = 5
    thick = 1
    town = None

    def build(self):

        if self.town is None:
            raise RuntimeError("Town is not defined")

        # Create a wall around the town: 4 walls are needed

        # The height is the same always
        wall_init_y = self.position.y

        wall_width = self.town.houses * (self.town.house_width + self.town.space) - self.town.space \
                     + 2 * self.wall_space

        wall_length = self.town.house_length + 2 * self.wall_space

        # First wall
        wall1_init_x = self.position.x
        wall1_init_z = self.position.z
        wall_pos = mcpi.vec3.Vec3(wall1_init_x, wall_init_y, wall1_init_z)
        wall = Wall(self.block, wall_pos, self.server)
        wall.height = self.height
        wall.length = self.thick
        wall.width = wall_width + self.thick
        wall.build()

        # Second wall
        wall2_init_x = wall1_init_x + (self.thick-1)
        wall2_init_z = wall1_init_z + wall.width
        wall_pos = mcpi.vec3.Vec3(wall2_init_x, wall_init_y, wall2_init_z)
        wall = Wall(self.block, wall_pos, self.server)
        wall.height = self.height
        wall.length = -(wall_length+2*self.thick-1)
        wall.width = self.thick
        wall.build()

        # Third wall
        wall3_init_x = wall2_init_x + (wall.length-1)
        wall3_init_z = wall2_init_z
        wall_pos = mcpi.vec3.Vec3(wall3_init_x, wall_init_y, wall3_init_z)
        wall = Wall(self.block, wall_pos, self.server)
        wall.height = self.height
        wall.length = self.thick
        wall.width = -(wall_width+self.thick-1)
        wall.build()

        # Fourth wall
        wall4_init_x = wall3_init_x
        wall4_init_z = wall3_init_z + (wall.width-1)
        wall_pos = mcpi.vec3.Vec3(wall4_init_x, wall_init_y, wall4_init_z)
        wall = Wall(self.block, wall_pos, self.server)
        wall.height = self.height
        wall.length = wall_length + self.thick + self.thick
        wall.width = self.thick
        wall.build()



import mcpi

from .object import Object
from .house import House


class Building(Object):

    floors = 10
    width = 10

    def build(self):
        init_x = self.position.x + self.width / 1.5
        init_y = self.position.y
        init_z = self.position.z - self.width / 2

        house_pos = mcpi.vec3.Vec3(init_x, init_y, init_z)

        for i in range(0, self.floors):
            house = House(self.block, house_pos, self.server)
            house_pos.y = house.height * i
            house.width = self.width
            house.build()



import mcpi


class Object:
    """ Base class for all objects in mcthings library """

    block = None
    block_empty = mcpi.block.AIR
    position = None
    server = None

    def __init__(self, block=None, position=None, server=None):
        self.block = block
        self.position = position
        self.server = server

    def build(self):
        pass

    def clean(self):
        block = self.block
        self.block = self.block_empty
        self.build()
        self.block = block



import mcpi

from .object import Object


class BlocksGallery(Object):

    MAX_BLOCK_NUMBER = 247

    def build(self):
        """
        Show all possible block types in a line
        :return:
        """

        for i in range(1, self.MAX_BLOCK_NUMBER):
            self.server.setBlock(self.position.x + i, self.position.y,
                                 self.position.z, mcpi.block.Block(i))



import mcpi

from .object import Object
from .house import House


class Town(Object):

    houses = 4
    house_width = 5
    house_length = 5
    house_height = 3
    space = 3  # space between houses

    def build(self):
        init_x = self.position.x
        init_y = self.position.y
        init_z = self.position.z

        house_pos = mcpi.vec3.Vec3(init_x, init_y, init_z)

        for i in range(0, self.houses):
            house = House(self.block, house_pos, self.server)
            house.width = self.house_width
            house.length = self.house_length
            house.height = self.house_height
            house.build()
            house_pos.z += self.house_width + self.space
